@@ -749,6 +749,12 @@ build_message (RBDAAPConnection *connection,
 #ifdef HAVE_LIBZ
 static void *g_zalloc_wrapper (voidpf opaque, uInt items, uInt size)
 {
+	if ((items != 0) && (size >= G_MAXUINT/items)) {
+		return Z_NULL;
+	}
+	if ((size != 0) && (items >= G_MAXUINT/size)) {
+		return Z_NULL;
+	}
 	return g_malloc0 (items * size);
 }
 
@@ -769,6 +775,14 @@ http_response_handler (SoupMessage *message,
 	int response_length = message->response.length;
 	const char *encoding_header = NULL;
 
+
+	if (response_length >= G_MAXUINT/4 - 1) {
+		/* If response_length is too big, 
+		 * the g_malloc (unc_size + 1) below would overflow 
+		 */
+		status = SOUP_STATUS_MALFORMED;
+	}
+
 	if (message->response_headers)
 		encoding_header = soup_message_get_header (message->response_headers, "Content-Encoding");
 
@@ -776,8 +790,9 @@ http_response_handler (SoupMessage *message,
 #ifdef HAVE_LIBZ
 		z_stream stream;
 		char *new_response;
-		int factor = 4;
-		int unc_size = response_length * factor;
+		unsigned int factor = 4;
+		unsigned int unc_size = response_length * factor;
+
 
 		stream.next_in = (unsigned char *)response;
 		stream.avail_in = response_length;
@@ -813,6 +828,10 @@ http_response_handler (SoupMessage *message,
 
 				factor *= 4;
 				unc_size = (response_length * factor);
+				/* unc_size can't grow bigger than 40KB, so
+				 * unc_size can't overflow, and this realloc
+				 * call is safe
+				 */
 				new_response = g_realloc (new_response, unc_size + 1);
 				stream.next_out = (unsigned char *)(new_response + stream.total_out);
 				stream.avail_out = unc_size - stream.total_out;
@@ -919,7 +938,7 @@ entry_set_string_prop (RhythmDB *db,
 	GValue value = {0,};
 	gchar *tmp;
 
-	if (str == NULL) {
+	if (str == NULL || *str == '\0') {
 		tmp = g_strdup (_("Unknown"));
 	} else {
 		tmp = g_strdup (str);
@@ -1170,27 +1189,46 @@ handle_song_listing (RBDAAPConnection *connection, guint status, GNode *structur
 			}
 		}
 
-//		if (connection->daap_version == 3.0) {
+		/*if (connection->daap_version == 3.0) {*/
 			uri = g_strdup_printf ("%s/databases/%d/items/%d.%s?session-id=%d", 
 					       priv->daap_base_uri, 
 					       priv->database_id, 
 					       item_id, format, 
 					       priv->session_id);
-//		} else {
-//		??FIXME??
-//		OLD ITUNES
-		// uri should be 
-		// "/databases/%d/items/%d.%s?session-id=%d&revision-id=%d";
-		// but its not going to work cause the other parts of the code 
-		// depend on the uri to have the ip address so that the
-		// RBDAAPSource can be found to ++request_id
-		// maybe just /dont/ support older itunes.  doesn't seem 
-		// unreasonable to me, honestly
-//		}
+		/*} else {*/
+		/* uri should be 
+		 * "/databases/%d/items/%d.%s?session-id=%d&revision-id=%d";
+		 * but its not going to work cause the other parts of the code 
+		 * depend on the uri to have the ip address so that the
+		 * RBDAAPSource can be found to ++request_id
+		 * maybe just /dont/ support older itunes.  doesn't seem 
+		 * unreasonable to me, honestly
+		 */
+		/*}*/
 		entry = rhythmdb_entry_new (priv->db, priv->db_type, uri);
+		if (entry == NULL) {
+			rb_debug ("cannot create entry for daap track %s", uri);
+			continue;
+		}
 		g_hash_table_insert (priv->item_id_to_uri, GINT_TO_POINTER (item_id), uri);
 
-		 /* track number */
+		/* year */
+		if (year != 0) {
+			GDate *date;
+			gulong julian;
+
+			/* create dummy date with given year */
+			date = g_date_new_dmy (1, G_DATE_JANUARY, year); 
+			julian = g_date_get_julian (date);
+			g_date_free (date);
+
+			g_value_init (&value, G_TYPE_ULONG);
+			g_value_set_ulong (&value,julian);
+			rhythmdb_entry_set_uninserted (priv->db, entry, RHYTHMDB_PROP_DATE, &value);
+			g_value_unset (&value);
+		} 
+
+		/* track number */
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value,(gulong)track_number);
 		rhythmdb_entry_set_uninserted (priv->db, entry, RHYTHMDB_PROP_TRACK_NUMBER, &value);
@@ -1427,19 +1465,23 @@ rb_daap_connection_logout (RBDAAPConnection *connection,
 			   gpointer user_data)
 {
 	RBDaapConnectionPrivate *priv = DAAP_CONNECTION_GET_PRIVATE (connection);
+	int old_state = priv->state;
 
 	if (priv->state == DAAP_LOGOUT)
 		return;
 	
-	/* FIXME what to do if we get asked to log out before login has completed? */
-	g_assert (priv->state == DAAP_DONE);
-
-	priv->callback = callback;
-	priv->callback_user_data = user_data;
-	priv->result = TRUE;
-	
 	priv->state = DAAP_LOGOUT;
-	rb_daap_connection_do_something (connection);
+
+	if (old_state <= DAAP_LOGIN) {
+		/* we haven't logged in yet, so we don't need to do anything here */
+		rb_daap_connection_state_done (connection, TRUE);
+	} else {
+		priv->callback = callback;
+		priv->callback_user_data = user_data;
+		priv->result = TRUE;
+		
+		rb_daap_connection_do_something (connection);
+	}
 }
 
 static void
@@ -1515,6 +1557,13 @@ rb_daap_connection_do_something (RBDAAPConnection *connection)
 				rb_daap_connection_do_something (connection);
 				return;
 			}
+
+			/* If the share went away while we were asking for the password,
+			 * don't bother trying to log in.
+			 */
+			if (priv->state != DAAP_GET_PASSWORD) {
+				return;
+			}
 		}
 
 		/* otherwise, fall through */
