@@ -28,9 +28,10 @@
 G_BEGIN_DECLS
 
 const char *	rb_file			(const char *filename);
-
 const char *	rb_dot_dir		(void);
 
+char *		rb_canonicalise_uri	(const char *uri);
+
 GnomeVFSResult	rb_uri_mkstemp		(const char *prefix, char **uri,
 					 GnomeVFSHandle **handle);
 
@@ -40,7 +41,7 @@ gboolean	rb_uri_exists		(const char *uri);
 char *		rb_uri_resolve_relative	(const char *uri);
 gboolean	rb_uri_is_readable	(const char *uri);
 gboolean	rb_uri_is_writable	(const char *uri);
-gboolean	rb_uri_is_iradio	(const char *uri);
+gboolean	rb_uri_is_local		(const char *uri);
 
 void		rb_uri_handle_recursively(const char *uri,
 					  GFunc func,
