@@ -1,4 +1,5 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of RhythmDB property GtkTreeModel
  *
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
@@ -24,7 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "gsequence.h"
@@ -147,7 +148,6 @@ struct RhythmDBPropertyModelPrivate
 	GPtrArray *query;
 
 	GSequence *properties;
-	GMemChunk *property_memchunk;
 	GHashTable *reverse_map;
 
 	RhythmDBPropertyModelEntry *all;
@@ -156,6 +156,8 @@ struct RhythmDBPropertyModelPrivate
 	guint syncing_id;
 };
 
+#define RHYTHMDB_PROPERTY_MODEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RHYTHMDB_TYPE_PROPERTY_MODEL, RhythmDBPropertyModelPrivate))
+
 enum
 {
 	PRE_ROW_DELETION,
@@ -237,6 +239,7 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
 
+	g_type_class_add_private (klass, sizeof (RhythmDBPropertyModelPrivate));
 }
 
 static void
@@ -272,8 +275,7 @@ rhythmdb_property_model_set_property (GObject *object,
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_RHYTHMDB:
 	{
 		model->priv->db = g_value_get_object (value);
@@ -286,8 +288,7 @@ rhythmdb_property_model_set_property (GObject *object,
 		break;
 	case PROP_PROP:
 		model->priv->propid = g_value_get_int (value);
-		switch (model->priv->propid)
-		{
+		switch (model->priv->propid) {
 		case RHYTHMDB_PROP_GENRE:
 			model->priv->sort_propid = RHYTHMDB_PROP_GENRE;
 			break;
@@ -297,6 +298,9 @@ rhythmdb_property_model_set_property (GObject *object,
 		case RHYTHMDB_PROP_ALBUM:
 			model->priv->sort_propid = RHYTHMDB_PROP_ALBUM;
 			break;
+		case RHYTHMDB_PROP_SUBTITLE:
+			model->priv->sort_propid = RHYTHMDB_PROP_SUBTITLE;
+			break;
 		case RHYTHMDB_PROP_TITLE:
 		case RHYTHMDB_PROP_LOCATION:
 			model->priv->sort_propid = RHYTHMDB_PROP_TITLE;
@@ -357,8 +361,7 @@ rhythmdb_property_model_get_property (GObject *object,
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_RHYTHMDB:
 		g_value_set_object (value, model->priv->db);
 		break;
@@ -380,18 +383,14 @@ rhythmdb_property_model_get_property (GObject *object,
 static void
 rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 {
-	model->priv = g_new0 (RhythmDBPropertyModelPrivate, 1);
-
-	model->priv->property_memchunk = g_mem_chunk_new ("RhythmDBPropertyModel property memchunk",
-							  sizeof (RhythmDBPropertyModelEntry),
-							  1024, G_ALLOC_AND_FREE);
+	model->priv = RHYTHMDB_PROPERTY_MODEL_GET_PRIVATE (model);
 
 	model->priv->stamp = g_random_int ();
 
 	model->priv->properties = g_sequence_new (NULL);
 	model->priv->reverse_map = g_hash_table_new (g_str_hash, g_str_equal);
 
-	model->priv->all = g_mem_chunk_alloc (model->priv->property_memchunk);
+	model->priv->all = g_new0 (RhythmDBPropertyModelEntry, 1);
 	model->priv->all->string = rb_refstring_new (_("All"));
 }
 
@@ -422,15 +421,12 @@ rhythmdb_property_model_finalize (GObject *object)
 		rb_refstring_unref (prop->sort_string);
 	}
 
-	g_mem_chunk_destroy (model->priv->property_memchunk);
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->properties);
 
 	if (model->priv->query_model)
 		g_object_unref (G_OBJECT (model->priv->query_model));
 
-	g_free (model->priv);
-
 	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
 
@@ -455,9 +451,11 @@ rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 RhythmDBPropertyModel *propmodel)
 {
 	RhythmDBEntry *entry = entry_from_tree_iter (model, iter);
-	rhythmdb_property_model_insert (propmodel, entry);
 
-	rhythmdb_property_model_sync (propmodel);
+	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN) == FALSE) {
+		rhythmdb_property_model_insert (propmodel, entry);
+		rhythmdb_property_model_sync (propmodel);
+	}
 }
 
 static void
@@ -466,13 +464,31 @@ rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 					 const GValue *new,
 					 RhythmDBPropertyModel *propmodel)
 {
-	if (prop != propmodel->priv->propid)
-		return;
+	if (prop == RHYTHMDB_PROP_HIDDEN) {
+		gboolean old_hidden = g_value_get_boolean (old);
+		gboolean new_hidden = g_value_get_boolean (new);
 
-	rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
-	rhythmdb_property_model_insert (propmodel, entry);
+		if (new_hidden == old_hidden)
+			return;
 
-	rhythmdb_property_model_sync (propmodel);
+		rb_debug ("recieving hidden-change notification for %s", entry->location);
+
+		if (new_hidden)
+			rhythmdb_property_model_delete (propmodel, entry);
+		else
+			rhythmdb_property_model_insert (propmodel, entry);
+
+		rhythmdb_property_model_sync (propmodel);
+	} else {	
+		if (prop != propmodel->priv->propid)
+			return;
+		if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN))
+			return;
+
+		rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
+		rhythmdb_property_model_insert (propmodel, entry);
+		rhythmdb_property_model_sync (propmodel);
+	}
 }
 
 static void
@@ -480,9 +496,10 @@ rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 					  RhythmDBEntry *entry,
 					  RhythmDBPropertyModel *propmodel)
 {
-	rhythmdb_property_model_delete (propmodel, entry);
-
-	rhythmdb_property_model_sync (propmodel);
+	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN) == FALSE) {
+		rhythmdb_property_model_delete (propmodel, entry);
+		rhythmdb_property_model_sync (propmodel);
+	}
 }
 
 static gint
@@ -507,6 +524,8 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 
 	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
 
+	model->priv->all->refcount++;
+
 	if ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr))) {
 		prop = g_sequence_ptr_get_data (ptr);
 		prop->refcount++;
@@ -514,7 +533,7 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 		return;
 	}
 
-	prop = g_mem_chunk_alloc (model->priv->property_memchunk);
+	prop = g_new0 (RhythmDBPropertyModelEntry, 1);
 	prop->string = rb_refstring_new (propstr);
 	prop->sort_string = rb_refstring_new (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
 	prop->refcount = 1;
@@ -554,6 +573,8 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 
 	g_assert ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr)));
 
+	model->priv->all->refcount--;
+
 	prop = g_sequence_ptr_get_data (ptr);
 	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
 	prop->refcount--;
@@ -571,7 +592,7 @@ rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
 	g_hash_table_remove (model->priv->reverse_map, propstr);
 	rb_refstring_unref (prop->string);
 	rb_refstring_unref (prop->sort_string);
-	g_mem_chunk_free (model->priv->property_memchunk, prop);
+	g_free (prop);
 	return;
 }
 
@@ -607,12 +628,13 @@ rhythmdb_property_model_get_n_columns (GtkTreeModel *tree_model)
 static GType
 rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
 {
-	switch (index)
-	{
-	case 0:
+	switch (index) {
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 		return G_TYPE_STRING;
-	case 1:
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 		return G_TYPE_BOOLEAN;
+	case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+		return G_TYPE_UINT;
 	default:
 		g_assert_not_reached ();
 		return G_TYPE_INVALID;
@@ -621,7 +643,7 @@ rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index)
 
 static gboolean
 rhythmdb_property_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
-			       GtkTreePath *path)
+				  GtkTreePath *path)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	guint index;
@@ -649,7 +671,7 @@ rhythmdb_property_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
 
 static GtkTreePath *
 rhythmdb_property_model_get_path (GtkTreeModel *tree_model,
-			       GtkTreeIter  *iter)
+				  GtkTreeIter  *iter)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	GtkTreePath *path;
@@ -677,32 +699,38 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 	g_return_if_fail (model->priv->stamp == iter->stamp);
 
 	if (iter->user_data == model->priv->all) {
-		switch (column)
-		{
-		case 0:
+		switch (column) {
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
 			g_value_set_string (value, rb_refstring_get (model->priv->all->string));
 			break;
-		case 1:
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 			g_value_init (value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (value, TRUE);
 			break;
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+			g_value_init (value, G_TYPE_UINT);
+			g_value_set_uint (value, model->priv->all->refcount);
+			break;
 		default:
 			g_assert_not_reached ();
 		}
 	} else {
 		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (iter->user_data);
 		
-		switch (column)
-		{
-		case 0:
+		switch (column) {
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE:
 			g_value_init (value, G_TYPE_STRING);
 			g_value_set_string (value, rb_refstring_get (prop->string));
 			break;
-		case 1:
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY:
 			g_value_init (value, G_TYPE_BOOLEAN);
 			g_value_set_boolean (value, prop == model->priv->all);
 			break;
+		case RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER:
+			g_value_init (value, G_TYPE_UINT);
+			g_value_set_uint (value, prop->refcount);
+			break;
 		default:
 			g_assert_not_reached ();
 		}
@@ -711,7 +739,7 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 
 static gboolean
 rhythmdb_property_model_iter_next (GtkTreeModel  *tree_model,
-				GtkTreeIter   *iter)
+				   GtkTreeIter   *iter)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 
@@ -765,8 +793,8 @@ rhythmdb_property_model_iter_n_children (GtkTreeModel *tree_model,
 
 static gboolean
 rhythmdb_property_model_iter_nth_child (GtkTreeModel *tree_model,
-				     GtkTreeIter *iter, GtkTreeIter *parent,
-				     gint n)
+					GtkTreeIter *iter, GtkTreeIter *parent,
+					gint n)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
 	GSequencePtr child;
@@ -843,16 +871,20 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (dragsource);
 	guint target;
 	GtkTargetList *drag_target_list;
+	GCompareDataFunc sort_func = NULL;
 
 	switch (model->priv->propid) {
 	case RHYTHMDB_PROP_GENRE:
 		drag_target_list = rhythmdb_property_model_genre_drag_target_list;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_genre_sort_func;
 		break;
 	case RHYTHMDB_PROP_ALBUM:
 		drag_target_list = rhythmdb_property_model_album_drag_target_list;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_album_sort_func;
 		break;
 	case RHYTHMDB_PROP_ARTIST:
 		drag_target_list = rhythmdb_property_model_artist_drag_target_list;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_artist_sort_func;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -866,13 +898,16 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 
 	if (target == TARGET_URIS) {
 		RhythmDB *db = model->priv->db;
- 		GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(db));
+ 		GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty (db));
  		GString* reply = g_string_new ("");
  		GtkTreeIter iter;
  		gboolean is_all;
  		struct QueryModelCbStruct tmp;
 		GtkTreePath *path;
 
+		rhythmdb_query_model_set_sort_order (RHYTHMDB_QUERY_MODEL (query_model),
+						     sort_func, 0, FALSE);
+
 		rb_debug ("getting drag data as uri list");
 		/* check if first selected row is 'All' */
 		path = gtk_tree_row_reference_get_path (paths->data);
@@ -1012,15 +1047,8 @@ rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *
 static gboolean
 rhythmdb_property_model_perform_sync (RhythmDBPropertyModel *model)
 {
-	int count = g_sequence_get_length (model->priv->properties);
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	char *s;
-
-	rb_refstring_unref (model->priv->all->string);
-	s = g_strdup_printf (_("All (%d)"), count);
-	model->priv->all->string = rb_refstring_new (s);
-	g_free (s);
 
 	iter.stamp = model->priv->stamp;
 	iter.user_data = model->priv->all;
