@@ -25,7 +25,6 @@
 #include <gtk/gtkactiongroup.h>
 
 #include "rb-source.h"
-#include "rb-player.h"
 #include "rhythmdb.h"
 
 #ifndef __RB_SHELL_PLAYER_H
@@ -71,6 +70,7 @@ typedef struct
 	void (*playing_source_changed) (RBShellPlayer *player, RBSource *source);
 	void (*playing_uri_changed) (RBShellPlayer *player, const char *uri);
 	void (*playing_song_changed) (RBShellPlayer *player, RhythmDBEntry *entry);
+	void (*playing_from_queue) (RBShellPlayer *player, gboolean from_queue);
 } RBShellPlayerClass;
 
 GType			rb_shell_player_get_type   (void);
@@ -105,8 +105,6 @@ gboolean		rb_shell_player_set_playing_time(RBShellPlayer *player,
 void			rb_shell_player_seek		(RBShellPlayer *player, long offset);
 long			rb_shell_player_get_playing_song_duration (RBShellPlayer *player);
 
-RBPlayer *		rb_shell_player_get_mm_player	(RBShellPlayer *shell_player);
-
 gboolean		rb_shell_player_get_playing	(RBShellPlayer *shell_player,
 							 gboolean *playing,
 							 GError **error);
@@ -128,6 +126,27 @@ RhythmDBEntry *         rb_shell_player_get_playing_entry (RBShellPlayer *player
 
 void			rb_shell_player_toggle_mute	(RBShellPlayer *player);
 
+gboolean		rb_shell_player_set_volume	(RBShellPlayer *player,
+							 gdouble volume,
+							 GError **error);
+
+gboolean		rb_shell_player_get_volume	(RBShellPlayer *player,
+							 gdouble *volume,
+							 GError **error);
+
+gboolean		rb_shell_player_set_volume_relative (RBShellPlayer *player,
+							 gdouble volume,
+							 GError **error);
+
+gboolean		rb_shell_player_set_mute	(RBShellPlayer *player,
+							 gboolean mute,
+							 GError **error);
+
+gboolean		rb_shell_player_get_mute	(RBShellPlayer *player,
+							 gboolean *mute,
+							 GError **error);
+
+
 #ifdef HAVE_ACME
 gboolean		rb_shell_player_handle_key	(RBShellPlayer *player, guint keyval);
 #endif
