@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of local file source object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
@@ -21,11 +22,12 @@
  */
 
 #include <config.h>
+#include <string.h>
+
 #include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <glade/glade.h>
-#include <string.h>
 
 #include "rb-source.h"
 #include "rb-library-source.h"
@@ -45,6 +47,7 @@
 #include "rb-song-info.h"
 #include "rb-search-entry.h"
 #include "rb-preferences.h"
+#include "rb-shell-preferences.h"
 
 typedef enum
 {
@@ -89,6 +92,8 @@ static void albums_selected_cb (RBPropertyView *propview, GList *albums,
 			       RBLibrarySource *libsource);
 static void albums_selection_reset_cb (RBPropertyView *propview, RBLibrarySource *libsource);
 static void songs_view_sort_order_changed_cb (RBEntryView *view, RBLibrarySource *source);
+static void rb_library_source_library_location_cb (GtkFileChooser *chooser,
+						   RBLibrarySource *source);
 
 static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
@@ -105,19 +110,25 @@ static void rb_library_source_browser_views_changed (GConfClient *client,
 						     guint cnxn_id,
 						     GConfEntry *entry,
 						     RBLibrarySource *source);
+static void rb_library_source_library_location_changed (GConfClient *client,
+							guint cnxn_id,
+							GConfEntry *entry,
+							RBLibrarySource *source);
+static void rb_library_source_prefs_update (RBShellPreferences *prefs,
+					    RBLibrarySource *source);
 static void rb_library_source_state_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_preferences_sync (RBLibrarySource *source);
 /* source methods */
-static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static GList *impl_get_extra_views (RBSource *source);
 static void impl_delete (RBSource *source);
+static void impl_move_to_trash (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_reset_filters (RBSource *source);
-static GtkWidget *impl_get_config_widget (RBSource *source);
+static GtkWidget *impl_get_config_widget (RBSource *source, RBShellPreferences *prefs);
 static void impl_song_properties (RBSource *source);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_show_popup (RBSource *source);
@@ -133,6 +144,8 @@ static void songs_view_drag_data_received_cb (GtkWidget *widget,
 					      GtkSelectionData *data,
 					      guint info, guint time,
 					      RBLibrarySource *source);
+static void rb_library_source_watch_toggled_cb (GtkToggleButton *button,
+						RBLibrarySource *source);
 
 
 #define CONF_UI_LIBRARY_DIR CONF_PREFIX "/ui/library"
@@ -144,8 +157,6 @@ static void songs_view_drag_data_received_cb (GtkWidget *widget,
 
 struct RBLibrarySourcePrivate
 {
-	gboolean disposed;
-	
 	RhythmDB *db;
 	
 	GtkWidget *browser;
@@ -162,28 +173,27 @@ struct RBLibrarySourcePrivate
 
 	gboolean lock;
 
-	char *artist;
-	char *album;
-
 	RhythmDBQueryModel *cached_all_query;
 	RhythmDBPropertyModel *cached_genres_model;
 	RhythmDBPropertyModel *cached_artists_model;
 	RhythmDBPropertyModel *cached_albums_model;
-	char *cached_sorting_type;
 	
 	RhythmDBQueryModel *model;
-	RhythmDBQueryModel *active_query;
-	RBLibraryQueryType query_type;
 	char *search_text;
 	GList *selected_genres;
 	GList *selected_artists;
 	GList *selected_albums;
 	
 	gboolean loading_prefs;
+	RBShellPreferences *shell_prefs;
 
 	GtkActionGroup *action_group;
 	GtkWidget *config_widget;
 	GSList *browser_views_group;
+
+	GtkFileChooser *library_location_widget;
+	GtkWidget *watch_library_check;
+	gboolean library_location_change_pending, library_location_handle_pending;
 	
 	RhythmDBEntryType entry_type;
 
@@ -193,8 +203,11 @@ struct RBLibrarySourcePrivate
 	guint state_browser_notify_id;
 	guint state_sorting_notify_id;
 	guint browser_view_notify_id;
+	guint library_location_notify_id;
 };
 
+#define RB_LIBRARY_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LIBRARY_SOURCE, RBLibrarySourcePrivate))
+
 static GtkActionEntry rb_library_source_actions [] =
 {
 	{ "LibrarySrcChooseGenre", NULL, N_("Browse This _Genre"), NULL,
@@ -218,36 +231,8 @@ enum
 	PROP_SORTING_KEY
 };
 
-static GObjectClass *parent_class = NULL;
 
-GType
-rb_library_source_get_type (void)
-{
-	static GType rb_library_source_type = 0;
-
-	if (rb_library_source_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBLibrarySourceClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_library_source_class_init,
-			NULL,
-			NULL,
-			sizeof (RBLibrarySource),
-			0,
-			(GInstanceInitFunc) rb_library_source_init
-		};
-
-		rb_library_source_type = g_type_register_static (RB_TYPE_SOURCE,
-								 "RBLibrarySource",
-								 &our_info, 0);
-
-	}
-
-	return rb_library_source_type;
-}
+G_DEFINE_TYPE (RBLibrarySource, rb_library_source, RB_TYPE_SOURCE)
 
 static void
 rb_library_source_class_init (RBLibrarySourceClass *klass)
@@ -255,8 +240,6 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->dispose = rb_library_source_dispose;
 	object_class->finalize = rb_library_source_finalize;
 	object_class->constructor = rb_library_source_constructor;
@@ -264,7 +247,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	object_class->set_property = rb_library_source_set_property;
 	object_class->get_property = rb_library_source_get_property;
 
-	source_class->impl_get_status = impl_get_status;
+	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_get_browser_key = impl_get_browser_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
@@ -278,12 +261,17 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
+	source_class->impl_move_to_trash = impl_move_to_trash;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
 
 	klass->impl_get_paned_key = impl_get_paned_key;
+	klass->impl_has_first_added_column = (RBLibrarySourceFeatureFunc) rb_true_function;
+	klass->impl_has_drop_support = (RBLibrarySourceFeatureFunc) rb_true_function;
 
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
@@ -310,6 +298,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 							      CONF_STATE_LIBRARY_SORTING,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_type_class_add_private (klass, sizeof (RBLibrarySourcePrivate));
 }
 
 static void
@@ -387,6 +376,16 @@ rb_library_source_browser_views_changed (GConfClient *client,
 	update_browser_views_visibility (source);
 }
 
+static void
+rb_library_source_library_location_changed (GConfClient *client,
+					    guint cnxn_id,
+					    GConfEntry *entry,
+					    RBLibrarySource *source)
+{
+	if (source->priv->config_widget)
+		rb_library_source_preferences_sync (source);
+}
+
 static void
 rb_library_source_ui_prefs_sync (RBLibrarySource *source)
 {
@@ -407,7 +406,7 @@ rb_library_source_ui_pref_changed (GConfClient *client,
 static void
 rb_library_source_init (RBLibrarySource *source)
 {
-	source->priv = g_new0 (RBLibrarySourcePrivate, 1);
+	source->priv = RB_LIBRARY_SOURCE_GET_PRIVATE (source);
 
 	/* Drag'n'Drop */
 
@@ -422,11 +421,20 @@ rb_library_source_dispose (GObject *object)
 	RBLibrarySource *source;
 	source = RB_LIBRARY_SOURCE (object);
 
-	if (source->priv->disposed)
-		return;
-	source->priv->disposed = TRUE;
+	if (source->priv->shell_prefs) {
+		g_object_unref (source->priv->shell_prefs);
+		source->priv->shell_prefs = NULL;
+	}
 
-	g_object_unref (source->priv->db);
+	if (source->priv->db) {
+		g_object_unref (source->priv->db);
+		source->priv->db = NULL;
+	}
+
+	if (source->priv->search_text) {
+		g_free (source->priv->search_text);
+		source->priv->search_text = NULL;
+	}
 }
 
 static void
@@ -445,21 +453,17 @@ rb_library_source_finalize (GObject *object)
 	
 	eel_gconf_notification_remove (source->priv->ui_dir_notify_id);
 	eel_gconf_notification_remove (source->priv->browser_view_notify_id);
+	eel_gconf_notification_remove (source->priv->library_location_notify_id);
 	eel_gconf_notification_remove (source->priv->state_browser_notify_id);
 	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
 	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
 
 	g_free (source->priv->sorting_key);
-	g_free (source->priv->artist);
-	g_free (source->priv->album);
 
-	g_free (source->priv->cached_sorting_type);
 	if (source->priv->cached_all_query)
 		g_object_unref (G_OBJECT (source->priv->cached_all_query));
 
-	g_free (source->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_library_source_parent_class)->finalize (object);
 }
 
 static void
@@ -513,20 +517,19 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 {
 	RBLibrarySource *source;
 	RBLibrarySourceClass *klass;
-	GObjectClass *parent_class;  
 	RBShell *shell;
+	GObject *shell_player;
 
 	klass = RB_LIBRARY_SOURCE_CLASS (g_type_class_peek (RB_TYPE_LIBRARY_SOURCE));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-
-	source = RB_LIBRARY_SOURCE (parent_class->constructor (type, n_construct_properties,
-							       construct_properties));
+	source = RB_LIBRARY_SOURCE (G_OBJECT_CLASS (rb_library_source_parent_class)
+			->constructor (type, n_construct_properties, construct_properties));
 
 	register_action_group (source);
 
 	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
 	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	shell_player = rb_shell_get_player (shell);
 	g_object_unref (G_OBJECT (shell));
 
 	source->priv->paned = gtk_vpaned_new ();
@@ -534,19 +537,22 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	source->priv->browser = gtk_hbox_new (TRUE, 5);
 
 	/* set up songs tree view */
-	source->priv->songs = rb_entry_view_new (source->priv->db, source->priv->sorting_key,
+	source->priv->songs = rb_entry_view_new (source->priv->db, shell_player,
+						 source->priv->sorting_key,
 						 TRUE, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_GENRE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ALBUM);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_YEAR);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_DURATION);
 /* 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_QUALITY); */
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
+	if (rb_library_source_has_first_added_column (source))
+		rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
 
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
 				 G_CALLBACK (rb_library_source_songs_show_popup_cb), source, 0);
@@ -564,10 +570,14 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 				 "drag_data_received",
 				 G_CALLBACK (songs_view_drag_data_received_cb),
 				 source, 0);
-	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs),
-			   GTK_DEST_DEFAULT_ALL,
-			   songs_view_drag_types, 1,
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE);	/* really accept move actions? */
+
+	/* only add drop support for the Library, subclasses can add it themselves */	
+	if (rb_library_source_has_drop_support (source)) {
+		gtk_drag_dest_set (GTK_WIDGET (source->priv->songs),
+				   GTK_DEST_DEFAULT_ALL,
+				   songs_view_drag_types, 1,
+				   GDK_ACTION_COPY | GDK_ACTION_MOVE);	/* really accept move actions? */
+	}
 
 	/* set up genres treeview */
 	source->priv->genres = rb_property_view_new (source->priv->db, RHYTHMDB_PROP_GENRE,
@@ -653,6 +663,9 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	source->priv->browser_view_notify_id =
 		eel_gconf_notification_add (CONF_UI_LIBRARY_BROWSER_VIEWS,
 				    (GConfClientNotifyFunc) rb_library_source_browser_views_changed, source);
+	source->priv->library_location_notify_id =
+		eel_gconf_notification_add (CONF_LIBRARY_LOCATION,
+				    (GConfClientNotifyFunc) rb_library_source_library_location_changed, source);
 
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
 	return G_OBJECT (source);
@@ -752,11 +765,11 @@ string_list_equal (GList *a, GList *b)
 		return FALSE;
 
 	for (sorted_a_keys = NULL; a; a = a->next) {
-		sorted_a_keys = g_list_append (sorted_a_keys,
+		sorted_a_keys = g_list_prepend (sorted_a_keys,
 					       g_utf8_collate_key (a->data, -1));
 	}
 	for (sorted_b_keys = NULL; b; b = b->next) {
-		sorted_b_keys = g_list_append (sorted_b_keys,
+		sorted_b_keys = g_list_prepend (sorted_b_keys,
 					       g_utf8_collate_key (b->data, -1));
 	}
 	sorted_a_keys = g_list_sort (sorted_a_keys, (GCompareFunc) strcmp);
@@ -867,14 +880,6 @@ rb_library_source_cmd_choose_album (GtkAction *action,
 	g_object_unref (source);
 }
 
-static void
-rb_library_source_sync_query (RBLibrarySource *source)
-{
-	GPtrArray *query = construct_query_from_selection (source);
-	g_object_set (G_OBJECT (source->priv->model), "query", query, NULL);
-	rhythmdb_query_free (query);
-}
-
 static void
 genres_selected_cb (RBPropertyView *propview, GList *genres,
 		   RBLibrarySource *libsource)
@@ -903,7 +908,7 @@ genres_selection_reset_cb (RBPropertyView *propview, RBLibrarySource *libsource)
 	g_list_foreach (libsource->priv->selected_genres, (GFunc) g_free, NULL);
 	g_list_free (libsource->priv->selected_genres);
 	libsource->priv->selected_genres = NULL;
-	rb_library_source_sync_query (libsource);
+	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_GENRE);
 }
 
 static void
@@ -935,7 +940,7 @@ artists_selection_reset_cb (RBPropertyView *propview, RBLibrarySource *libsource
 	g_list_foreach (libsource->priv->selected_artists, (GFunc) g_free, NULL);
 	g_list_free (libsource->priv->selected_artists);
 	libsource->priv->selected_artists = NULL;
-	rb_library_source_sync_query (libsource);
+	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_ALBUM);
 }
 
 static void
@@ -966,27 +971,14 @@ albums_selection_reset_cb (RBPropertyView *propview, RBLibrarySource *libsource)
 	g_list_foreach (libsource->priv->selected_albums, (GFunc) g_free, NULL);
 	g_list_free (libsource->priv->selected_albums);
 	libsource->priv->selected_albums = NULL;
-	rb_library_source_sync_query (libsource);
+	rb_library_source_do_query (libsource, RB_LIBRARY_QUERY_TYPE_ALBUM);
 }
 
 static void
 songs_view_sort_order_changed_cb (RBEntryView *view, RBLibrarySource *source)
 {
 	rb_debug ("sort order changed");
-	rb_entry_view_set_resorting (view);
-	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_SEARCH);
-}
-
-static char *
-impl_get_status (RBSource *asource)
-{
-	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	gchar *status;
-
-	status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
-						 rb_entry_view_get_duration (source->priv->songs),
-						 rb_entry_view_get_total_size (source->priv->songs));
-	return status;
+	rb_entry_view_resort_model (view);
 }
 
 static const char *
@@ -1048,9 +1040,9 @@ impl_get_extra_views (RBSource *asource)
 	GList *ret;
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 
-	ret = g_list_append (NULL, source->priv->genres);
-	ret = g_list_append (ret, source->priv->artists);
-	ret = g_list_append (ret, source->priv->albums);
+	ret = g_list_prepend (NULL, source->priv->albums);
+	ret = g_list_prepend (ret, source->priv->artists);
+	ret = g_list_prepend (ret, source->priv->genres);
 	return ret;
 }
 
@@ -1090,18 +1082,18 @@ impl_reset_filters (RBSource *asource)
 }
   
 static GtkWidget *
-impl_get_config_widget (RBSource *asource)
+impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
 	GtkWidget *tmp;
-	GtkWidget *browser_views_label;
-	PangoAttrList *pattrlist;
-	PangoAttribute *attr;
 	GladeXML *xml;
 
 	if (source->priv->config_widget)
 		return source->priv->config_widget;
-
+	
+	g_object_ref (G_OBJECT (prefs));
+	source->priv->shell_prefs = prefs;
+	
 	xml = rb_glade_xml_new ("library-prefs.glade", "library_vbox", source);
 	source->priv->config_widget =
 		glade_xml_get_widget (xml, "library_vbox");
@@ -1110,18 +1102,28 @@ impl_get_config_widget (RBSource *asource)
 		g_slist_reverse (g_slist_copy (gtk_radio_button_get_group 
 					       (GTK_RADIO_BUTTON (tmp))));
 
-	pattrlist = pango_attr_list_new ();
-	attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-	attr->start_index = 0;
-	attr->end_index = G_MAXINT;
-	pango_attr_list_insert (pattrlist, attr);
-	browser_views_label = glade_xml_get_widget (xml, "browser_views_label");
-	gtk_label_set_attributes (GTK_LABEL (browser_views_label), pattrlist);
-	pango_attr_list_unref (pattrlist);
+	source->priv->library_location_widget =
+		(GtkFileChooser*) glade_xml_get_widget (xml, "library_location_chooser");
+	source->priv->watch_library_check = glade_xml_get_widget (xml, "watch_library_check");
 
+	rb_glade_boldify_label (xml, "browser_views_label");
+	rb_glade_boldify_label (xml, "library_location_label");
 	g_object_unref (G_OBJECT (xml));
 	
 	rb_library_source_preferences_sync (source);
+	g_signal_connect (G_OBJECT (source->priv->library_location_widget),
+			  "selection-changed",
+			  G_CALLBACK (rb_library_source_library_location_cb),
+			  asource);
+	g_signal_connect (G_OBJECT (source->priv->shell_prefs),
+			  "hide",
+			  G_CALLBACK (rb_library_source_prefs_update),
+			  asource);
+	g_signal_connect (G_OBJECT (source->priv->watch_library_check),
+			  "toggled",
+			  G_CALLBACK (rb_library_source_watch_toggled_cb),
+			  asource);
+
 	return source->priv->config_widget;
 }
 
@@ -1135,6 +1137,26 @@ rb_library_source_preferences_sync (RBLibrarySource *source)
 	list = g_slist_nth (source->priv->browser_views_group,
 			    eel_gconf_get_integer (CONF_UI_LIBRARY_BROWSER_VIEWS));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (list->data), TRUE);
+
+	list = eel_gconf_get_string_list (CONF_LIBRARY_LOCATION);
+
+	if (g_slist_length (list) == 1) {
+		/* the uri might be missing the trailing slash */
+		gchar *s = g_strconcat (list->data, G_DIR_SEPARATOR_S, NULL);
+		gtk_file_chooser_set_uri (source->priv->library_location_widget, s);
+		rb_debug ("syncing library location %s", s);
+		g_free (s);
+	} else {
+		/* either no or multiple folders are chosen. make the widget blank*/
+		/*gtk_file_chooser_set_uri (source->priv->library_location_widget,
+					  "");*/
+	}
+
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (source->priv->watch_library_check),
+				      eel_gconf_get_boolean (CONF_MONITOR_LIBRARY));
 }
 
 void
@@ -1155,12 +1177,29 @@ static void
 impl_delete (RBSource *asource)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	GList *l;
+	GList *sel, *tem;
 
-	for (l = rb_entry_view_get_selected_entries (source->priv->songs); l != NULL; l = g_list_next (l)) {
-		rhythmdb_entry_delete (source->priv->db, l->data);
+	sel = rb_entry_view_get_selected_entries (source->priv->songs);
+	for (tem = sel; tem != NULL; tem = tem->next) {
+		rhythmdb_entry_delete (source->priv->db, tem->data);
 		rhythmdb_commit (source->priv->db);
 	}
+	g_list_free (sel);
+}
+
+static void
+impl_move_to_trash (RBSource *asource)
+{
+	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
+	GList *sel, *tem;
+
+	sel = rb_entry_view_get_selected_entries (source->priv->songs);
+	for (tem = sel; tem != NULL; tem = tem->next) {
+		rhythmdb_entry_move_to_trash (source->priv->db,
+					       (RhythmDBEntry *) tem->data);
+		rhythmdb_commit (source->priv->db);
+	}
+	g_list_free (sel);
 }
 
 static void
@@ -1171,7 +1210,7 @@ impl_song_properties (RBSource *asource)
 
 	g_return_if_fail (source->priv->songs != NULL);
         
- 	song_info = rb_song_info_new (source->priv->songs); 
+ 	song_info = rb_song_info_new (asource, NULL);
 
         g_return_if_fail (song_info != NULL);
 
@@ -1311,7 +1350,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	uri_list = NULL;
 
 	for (i = list; i != NULL; i = g_list_next (i))
-		uri_list = g_list_append (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+		uri_list = g_list_prepend (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
 
 	gnome_vfs_uri_list_free (list);
 
@@ -1351,6 +1390,23 @@ rb_library_source_get_paned_key (RBLibrarySource *source)
 		return NULL;
 }
 
+gboolean
+rb_library_source_has_first_added_column (RBLibrarySource *source)
+{
+	RBLibrarySourceClass *klass = RB_LIBRARY_SOURCE_GET_CLASS (source);
+
+	return klass->impl_has_first_added_column (source);
+}
+
+gboolean
+rb_library_source_has_drop_support (RBLibrarySource *source)
+{
+	RBLibrarySourceClass *klass = RB_LIBRARY_SOURCE_GET_CLASS (source);
+
+	return klass->impl_has_drop_support (source);
+}
+
+
 static void
 push_multi_equals_query (RhythmDB *db, GPtrArray *query, guint propid, GList *items)
 {
@@ -1401,19 +1457,7 @@ construct_query_from_selection (RBLibrarySource *source)
 	if (source->priv->search_text) {
 		GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
 							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_GENRE_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_DISJUNCTION,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_ARTIST_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_DISJUNCTION,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_ALBUM_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_DISJUNCTION,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_TITLE_FOLDED,
+							    RHYTHMDB_PROP_SEARCH_MATCH,
 							    source->priv->search_text,
 							    RHYTHMDB_QUERY_END);
 		rhythmdb_query_append (source->priv->db,
@@ -1480,71 +1524,29 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 	RhythmDBPropertyModel *genre_model = NULL;
 	RhythmDBPropertyModel *artist_model = NULL;
 	RhythmDBPropertyModel *album_model = NULL;
-	GtkTreeModel *model;
 	GPtrArray *query;
-	gboolean is_all_query, sorting_matches;
-	const char *current_sorting_type;
-
-	/* Unlocked */
-	rb_debug ("preparing to read lock for query");
-
-	if (source->priv->active_query) {
-		rb_debug ("killing active query");
-		rhythmdb_query_model_cancel (RHYTHMDB_QUERY_MODEL (source->priv->active_query));
-	}
+	gboolean is_all_query;
+	gboolean do_new_query = TRUE;
 
 	is_all_query = (source->priv->selected_genres == NULL &&
 			source->priv->selected_artists == NULL &&	    
 			source->priv->selected_albums == NULL &&	    
 			source->priv->search_text == NULL);
-	current_sorting_type = rb_entry_view_get_sorting_type (source->priv->songs);
-	sorting_matches = source->priv->cached_sorting_type
-	&& !strcmp (source->priv->cached_sorting_type, current_sorting_type);
-	rb_debug ("current sorting: %s, match: %s", current_sorting_type, sorting_matches ? "TRUE" : "FALSE" );
 
-	if (is_all_query) {
-		if (sorting_matches) {
-			rb_debug ("cached query hit");
-			source->priv->model = source->priv->cached_all_query;
-			source->priv->active_query = NULL;
-			rb_entry_view_set_model (source->priv->songs,
-						 RHYTHMDB_QUERY_MODEL (source->priv->cached_all_query));
-			g_object_set (G_OBJECT (source->priv->cached_genres_model),
-				      "query-model", source->priv->cached_all_query, NULL);
-			g_object_set (G_OBJECT (source->priv->cached_artists_model),
-				      "query-model", source->priv->cached_all_query, NULL);
-			g_object_set (G_OBJECT (source->priv->cached_albums_model),
-				      "query-model", source->priv->cached_all_query, NULL);
-			rb_property_view_set_model (source->priv->genres,
-						    source->priv->cached_genres_model);
-			rb_property_view_set_model (source->priv->artists,
-						    source->priv->cached_artists_model);
-			rb_property_view_set_model (source->priv->albums,
-						    source->priv->cached_albums_model);
-			rb_entry_view_poll_model (source->priv->songs);
-			return;
-		} else if (source->priv->cached_all_query) {
-			rb_debug ("sorting mismatch, freeing cached query");
-			g_object_unref (source->priv->cached_all_query);
-			g_object_unref (source->priv->cached_genres_model);
-			g_object_unref (source->priv->cached_artists_model);
-			g_object_unref (source->priv->cached_albums_model);
-			g_free (source->priv->cached_sorting_type);
+	if ((source->priv->cached_all_query == NULL) || is_all_query ) {
+		if (source->priv->cached_all_query == NULL) {
+			rb_debug ("caching new query");
+			query_model = source->priv->model = 
+				source->priv->cached_all_query = rhythmdb_query_model_new_empty (source->priv->db);
+			source->priv->cached_genres_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_GENRE);
+			source->priv->cached_artists_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_ARTIST);
+			source->priv->cached_albums_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_ALBUM);
+		} else {
+			rb_debug ("using cached query");
+			query_model = source->priv->model = source->priv->cached_all_query;
+			do_new_query = FALSE;
 		}
-	}
-
-	source->priv->query_type = qtype;
-	rb_debug ("query type: %d", qtype);
-
-	if (source->priv->cached_all_query == NULL
-	    || (is_all_query && !sorting_matches)) {
-		rb_debug ("caching new query");
-		query_model = source->priv->active_query = source->priv->model = 
-			source->priv->cached_all_query = rhythmdb_query_model_new_empty (source->priv->db);
-		source->priv->cached_sorting_type = g_strdup (current_sorting_type);
-		source->priv->cached_genres_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_GENRE);
-		source->priv->cached_artists_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_ARTIST);
-		source->priv->cached_albums_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_ALBUM);
+		
 		g_object_set (G_OBJECT (source->priv->cached_genres_model),
 			      "query-model", source->priv->cached_all_query, NULL);
 		g_object_set (G_OBJECT (source->priv->cached_artists_model),
@@ -1559,7 +1561,7 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 					    source->priv->cached_albums_model);
 	} else {
 		rb_debug ("query is not special");
-		source->priv->active_query = source->priv->model = 
+		source->priv->model = 
 			query_model = rhythmdb_query_model_new_empty (source->priv->db);
 
 		if (source->priv->selected_genres == NULL) {
@@ -1585,6 +1587,7 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 			genre_model = rb_property_view_get_model (source->priv->genres);
 			g_object_set (G_OBJECT (genre_model), "query-model",
 				      query_model, NULL);
+			g_object_unref (G_OBJECT (genre_model));
 		case RB_LIBRARY_QUERY_TYPE_GENRE:
 			rb_debug ("resetting artist view");
 			g_list_foreach (source->priv->selected_artists,
@@ -1596,6 +1599,7 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 			artist_model = rb_property_view_get_model (source->priv->artists);
 			g_object_set (G_OBJECT (artist_model), "query-model",
 				      query_model, NULL);
+			g_object_unref (G_OBJECT (artist_model));
 		case RB_LIBRARY_QUERY_TYPE_ARTIST:
 			rb_debug ("resetting album view");
 			g_list_foreach (source->priv->selected_albums,
@@ -1607,24 +1611,28 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 			album_model = rb_property_view_get_model (source->priv->albums);
 			g_object_set (G_OBJECT (album_model), "query-model",
 				      query_model, NULL);
+			g_object_unref (G_OBJECT (album_model));
 		case RB_LIBRARY_QUERY_TYPE_ALBUM:
 		case RB_LIBRARY_QUERY_TYPE_SEARCH:
 			break;
 		}
 	}
 
-	model = GTK_TREE_MODEL (query_model);
+	rb_debug ("setting query model");
+	rb_entry_view_set_model (source->priv->songs, query_model);
+	g_object_set (RB_SOURCE (source), "query-model", query_model, NULL);
 
-	rb_debug ("setting empty model");
-	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (query_model));
-
-	query = construct_query_from_selection (source);
-	
-	rb_debug ("doing query");
-	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
+	if (do_new_query) {
+		query = construct_query_from_selection (source);
 		
-	rhythmdb_query_free (query);
-	g_object_unref (G_OBJECT (query_model));
+		rb_debug ("doing query");
+		rhythmdb_do_full_query_async_parsed (source->priv->db, GTK_TREE_MODEL (query_model), query);
+			
+		rhythmdb_query_free (query);
+	}
+	
+	if (!is_all_query)
+		g_object_unref (G_OBJECT (query_model));
 }
 
 static void
@@ -1639,3 +1647,72 @@ songs_view_drag_data_received_cb (GtkWidget *widget,
 	rb_source_receive_drag (RB_SOURCE (source), selection_data);
 }
 
+static gboolean
+rb_library_source_process_library_location_change (RBLibrarySource *source)
+{
+	GSList *list;
+	
+	if (!source->priv->library_location_change_pending)
+		return FALSE;
+
+	/* process the change */
+	list = gtk_file_chooser_get_uris (source->priv->library_location_widget);
+	eel_gconf_set_string_list (CONF_LIBRARY_LOCATION, list);
+
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+
+	source->priv->library_location_change_pending = FALSE;
+
+	return FALSE;
+}
+
+static void
+rb_library_source_prefs_update (RBShellPreferences *prefs,
+				RBLibrarySource *source)
+{
+	rb_library_source_process_library_location_change (source);
+}
+
+static gboolean
+rb_library_source_process_library_handle_selection (RBLibrarySource *source)
+{
+	GSList *list;
+	
+	if (!source->priv->library_location_handle_pending)
+		return FALSE;
+
+	/* this can't be processed immediately, because we sometimes get intemediate signals */
+	for (list = gtk_file_chooser_get_uris (source->priv->library_location_widget);
+	     list != NULL ; list = g_slist_next (list)) {
+		if ((strcmp (list->data, "file:///") == 0) ||
+		    (strcmp (list->data, "file://") == 0)) {
+			rb_error_dialog (GTK_WINDOW (source->priv->shell_prefs), _("Cannot select library location"),
+					 _("The root filesystem cannot be chosen as your library location. Please choose a different location"));
+		} else {
+			source->priv->library_location_change_pending = TRUE;
+		}
+	}
+
+	source->priv->library_location_handle_pending = FALSE;
+	return FALSE;
+}
+
+static void
+rb_library_source_library_location_cb (GtkFileChooser *chooser,
+				       RBLibrarySource *source)
+{
+	source->priv->library_location_handle_pending = TRUE;
+
+	g_idle_add ((GSourceFunc)rb_library_source_process_library_handle_selection, source);
+}
+
+static void
+rb_library_source_watch_toggled_cb (GtkToggleButton *button, RBLibrarySource *source)
+{
+	gboolean active;
+
+	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (source->priv->watch_library_check));
+	eel_gconf_set_boolean (CONF_MONITOR_LIBRARY, active);
+}
+
