@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of local song properties dialog
  *
  *  Copyright (C) 2002 Olivier Martin <omartin@ifrance.com>
@@ -26,12 +26,14 @@
  */
 
 #include <config.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtk.h>
-#include <glade/glade.h>
+
 #include <string.h>
 #include <time.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
@@ -42,6 +44,7 @@
 #include "rb-rating.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
+#include "rb-source.h"
 
 static void rb_song_info_class_init (RBSongInfoClass *klass);
 static void rb_song_info_init (RBSongInfo *song_info);
@@ -62,6 +65,7 @@ static void rb_song_info_response_cb (GtkDialog *dialog,
 				      int response_id,
 				      RBSongInfo *song_info);
 static void rb_song_info_populate_dialog (RBSongInfo *song_info);
+static void rb_song_info_populate_dialog_multiple (RBSongInfo *song_info);
 static void rb_song_info_update_duration (RBSongInfo *song_info);
 static void rb_song_info_update_location (RBSongInfo *song_info);
 static void rb_song_info_update_play_count (RBSongInfo *song_info);
@@ -76,8 +80,9 @@ static void rb_song_info_backward_clicked_cb (GtkWidget *button,
 					      RBSongInfo *song_info);
 static void rb_song_info_forward_clicked_cb (GtkWidget *button,
 					     RBSongInfo *song_info);
-static void rb_song_info_view_changed_cb (RBEntryView *entry_view,
-					  RBSongInfo *song_info);
+static void rb_song_info_query_model_changed_cb (GObject *source,
+						 GParamSpec *pspec,
+						 RBSongInfo *song_info);
 static void rb_song_info_rated_cb (RBRating *rating,
 				   double score,
 				   RBSongInfo *song_info);
@@ -87,7 +92,9 @@ static void rb_song_info_sync_entries (RBSongInfo *dialog);
 struct RBSongInfoPrivate
 {
 	RhythmDB *db;
+	RBSource *source;
 	RBEntryView *entry_view;
+	RhythmDBQueryModel *query_model;
 
 	/* information on the displayed song */
 	RhythmDBEntry *current_entry;
@@ -118,6 +125,8 @@ struct RBSongInfoPrivate
 	GtkWidget   *rating;
 };
 
+#define RB_SONG_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SONG_INFO, RBSongInfoPrivate))
+
 enum
 {
 	PRE_METADATA_CHANGE,
@@ -128,40 +137,13 @@ enum
 enum 
 {
 	PROP_0,
+	PROP_SOURCE,
 	PROP_ENTRY_VIEW
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rb_song_info_signals[LAST_SIGNAL] = { 0 };
 
-GType
-rb_song_info_get_type (void)
-{
-	static GType rb_song_info_type = 0;
-
-	if (rb_song_info_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBSongInfoClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_song_info_class_init,
-			NULL,
-			NULL,
-			sizeof (RBSongInfo),
-			0,
-			(GInstanceInitFunc) rb_song_info_init
-		};
-
-		rb_song_info_type = g_type_register_static (GTK_TYPE_DIALOG,
-							    "RBSongInfo",
-							    &our_info, 0);
-	}
-
-	return rb_song_info_type;
-}
+G_DEFINE_TYPE (RBSongInfo, rb_song_info, GTK_TYPE_DIALOG)
 
 static void
 rb_song_info_class_init (RBSongInfoClass *klass)
@@ -169,17 +151,22 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->set_property = rb_song_info_set_property;
 	object_class->get_property = rb_song_info_get_property;
 	object_class->constructor = rb_song_info_constructor;
 
 	widget_class->show = rb_song_info_show;
 
+	g_object_class_install_property (object_class,
+					 PROP_SOURCE,
+					 g_param_spec_object ("source",
+					                      "RBSource",
+					                      "RBSource object",
+					                      RB_TYPE_SOURCE,
+					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_VIEW,
-					 g_param_spec_object ("entry_view",
+					 g_param_spec_object ("entry-view",
 					                      "RBEntryView",
 					                      "RBEntryView object",
 					                      RB_TYPE_ENTRY_VIEW,
@@ -208,13 +195,15 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
+
+	g_type_class_add_private (klass, sizeof (RBSongInfoPrivate));
 }
 
 static void
 rb_song_info_init (RBSongInfo *song_info)
 {
 	/* create the dialog and some buttons backward - forward - close */
-	song_info->priv = g_new0 (RBSongInfoPrivate, 1);
+	song_info->priv = RB_SONG_INFO_GET_PRIVATE (song_info);
 
 	g_signal_connect_object (G_OBJECT (song_info),
 				 "response",
@@ -223,15 +212,6 @@ rb_song_info_init (RBSongInfo *song_info)
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (song_info), FALSE);
 
-	gtk_dialog_add_button (GTK_DIALOG (song_info),
-			       GTK_STOCK_CLOSE,
-			       GTK_RESPONSE_CLOSE);
-
-	gtk_dialog_set_default_response (GTK_DIALOG (song_info),
-					 GTK_RESPONSE_CLOSE);
-
-
-
 	gtk_container_set_border_width (GTK_CONTAINER (song_info), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (song_info)->vbox), 2);
 }
@@ -239,27 +219,12 @@ rb_song_info_init (RBSongInfo *song_info)
 static void
 rb_song_info_show (GtkWidget *widget)
 {
-	if (GTK_WIDGET_CLASS (parent_class)->show)
-		GTK_WIDGET_CLASS (parent_class)->show (widget);
+	if (GTK_WIDGET_CLASS (rb_song_info_parent_class)->show)
+		GTK_WIDGET_CLASS (rb_song_info_parent_class)->show (widget);
 
 	rb_song_info_update_playback_error (RB_SONG_INFO (widget));
 }
 
-static void
-rb_song_info_boldify_label (RBSongInfo *song_info, GladeXML *xml, 
-			    const char *label_str)
-{
-	gchar *str_final;
-	GtkWidget *label;
-	
-	/* We add now the Pango attributes (look at bug #99867 and #97061) */
-	label = glade_xml_get_widget (xml, label_str);
-	str_final = g_strdup_printf ("<b>%s</b>",
-				     gtk_label_get_label GTK_LABEL (label));
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
-	g_free (str_final);
-}
-
 static void
 rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 			       gboolean editable)
@@ -293,19 +258,15 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
 	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
 	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
-	song_info->priv->disc_cur     = glade_xml_get_widget (xml, "song_info_disc_cur");
-	song_info->priv->year		= glade_xml_get_widget (xml, "song_info_year");
 
-	rb_song_info_boldify_label (song_info, xml, "title_label");
-	rb_song_info_boldify_label (song_info, xml, "trackn_label");
-	rb_song_info_boldify_label (song_info, xml, "name_label");
-	rb_song_info_boldify_label (song_info, xml, "location_label");
-	rb_song_info_boldify_label (song_info, xml, "last_played_label");
-	rb_song_info_boldify_label (song_info, xml, "play_count_label");
-	rb_song_info_boldify_label (song_info, xml, "duration_label");
-	rb_song_info_boldify_label (song_info, xml, "bitrate_label");
-	rb_song_info_boldify_label (song_info, xml, "discn_label");
-	rb_song_info_boldify_label (song_info, xml, "year_label");
+	rb_glade_boldify_label (xml, "title_label");
+	rb_glade_boldify_label (xml, "trackn_label");
+	rb_glade_boldify_label (xml, "name_label");
+	rb_glade_boldify_label (xml, "location_label");
+	rb_glade_boldify_label (xml, "last_played_label");
+	rb_glade_boldify_label (xml, "play_count_label");
+	rb_glade_boldify_label (xml, "duration_label");
+	rb_glade_boldify_label (xml, "bitrate_label");
 
 	/* whenever you press a mnemonic, the associated GtkEntry's text gets highlighted */
 	g_signal_connect_object (G_OBJECT (song_info->priv->title),
@@ -340,7 +301,6 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 {
 	RBSongInfo *song_info;
 	RBSongInfoClass *klass;
-	GObjectClass *parent_class;  
 	GladeXML *xml;
 	GList *selected_entries;
 	GList *tem;
@@ -348,10 +308,8 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 
 	klass = RB_SONG_INFO_CLASS (g_type_class_peek (RB_TYPE_SONG_INFO));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	song_info = RB_SONG_INFO (parent_class->constructor (type,
-							     n_construct_properties,
-							     construct_properties));
+	song_info = RB_SONG_INFO (G_OBJECT_CLASS (rb_song_info_parent_class)
+			->constructor (type, n_construct_properties, construct_properties));
 
 	selected_entries = rb_entry_view_get_selected_entries (song_info->priv->entry_view);
 
@@ -394,13 +352,17 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	song_info->priv->artist = glade_xml_get_widget (xml, "song_info_artist");
 	song_info->priv->album = glade_xml_get_widget (xml, "song_info_album");
 	song_info->priv->genre = glade_xml_get_widget (xml, "song_info_genre");
+	song_info->priv->year = glade_xml_get_widget (xml, "song_info_year");
 	song_info->priv->playback_error_box = glade_xml_get_widget (xml, "song_info_error_box");
 	song_info->priv->playback_error_label = glade_xml_get_widget (xml, "song_info_error_label");
+	song_info->priv->disc_cur = glade_xml_get_widget (xml, "song_info_disc_cur");
 
-	rb_song_info_boldify_label (song_info, xml, "album_label");
-	rb_song_info_boldify_label (song_info, xml, "artist_label");
-	rb_song_info_boldify_label (song_info, xml, "genre_label");
-	rb_song_info_boldify_label (song_info, xml, "rating_label");
+	rb_glade_boldify_label (xml, "album_label");
+	rb_glade_boldify_label (xml, "artist_label");
+	rb_glade_boldify_label (xml, "genre_label");
+	rb_glade_boldify_label (xml, "year_label");
+	rb_glade_boldify_label (xml, "rating_label");
+	rb_glade_boldify_label (xml, "discn_label");
 
 	g_signal_connect_object (G_OBJECT (song_info->priv->artist),
 				 "mnemonic-activate",
@@ -414,6 +376,14 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 				 "mnemonic-activate",
 				 G_CALLBACK (rb_song_info_mnemonic_cb),
 				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->year),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->disc_cur),
+				 "mnemonic-activate",
+				 G_CALLBACK (rb_song_info_mnemonic_cb),
+				 NULL, 0);
 
 	/* this widget has to be customly created */
 	song_info->priv->rating = GTK_WIDGET (rb_rating_new ());
@@ -424,14 +394,26 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 			   song_info->priv->rating);
 
 	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->artist), editable);
-	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->album), editable);
-	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->genre), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->album), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->genre), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->year), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (song_info->priv->disc_cur), editable);
 
 	/* Finish construction */
-	if (song_info->priv->current_entry)
+	if (song_info->priv->current_entry) {
 		rb_song_info_construct_single (song_info, xml, editable);
-	else
+		rb_song_info_populate_dialog (song_info);
+	} else {
 		rb_song_info_construct_multiple (song_info, xml, editable);
+		rb_song_info_populate_dialog_multiple (song_info);
+	}
+
+	gtk_dialog_add_button (GTK_DIALOG (song_info),
+			       GTK_STOCK_CLOSE,
+			       GTK_RESPONSE_CLOSE);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (song_info),
+					 GTK_RESPONSE_CLOSE);
 
 	g_object_unref (G_OBJECT (xml));
 	return G_OBJECT (song_info);
@@ -449,9 +431,15 @@ rb_song_info_finalize (GObject *object)
 
 	g_return_if_fail (song_info->priv != NULL);
 
-	g_free (song_info->priv);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (song_info->priv->source),
+					      G_CALLBACK (rb_song_info_query_model_changed_cb),
+					      song_info);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	g_object_unref (G_OBJECT (song_info->priv->db));
+	g_object_unref (G_OBJECT (song_info->priv->source));
+	g_object_unref (G_OBJECT (song_info->priv->query_model));
+
+	G_OBJECT_CLASS (rb_song_info_parent_class)->finalize (object);
 }
 
 static void
@@ -464,18 +452,24 @@ rb_song_info_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_ENTRY_VIEW:
+	case PROP_SOURCE:
 	{
-		song_info->priv->entry_view = g_value_get_object (value);
-		g_object_get (G_OBJECT (song_info->priv->entry_view), "db",
+		song_info->priv->source = g_value_get_object (value);
+		g_object_ref (G_OBJECT (song_info->priv->source));
+		g_object_get (G_OBJECT (song_info->priv->source), 
+			      "query-model", &song_info->priv->query_model, NULL);
+		g_signal_connect_object (G_OBJECT (song_info->priv->source),
+					 "notify::query-model",
+					 G_CALLBACK (rb_song_info_query_model_changed_cb),
+					 song_info, 0);
+
+		g_object_get (G_OBJECT (song_info->priv->query_model), "db", 
 			      &song_info->priv->db, NULL);
-		g_signal_connect_object (G_OBJECT (song_info->priv->entry_view),
-					 "changed",
-					 G_CALLBACK (rb_song_info_view_changed_cb),
-					 song_info,
-					 G_CONNECT_AFTER);
 	}
 	break;
+	case PROP_ENTRY_VIEW:
+		song_info->priv->entry_view = g_value_get_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -492,6 +486,9 @@ rb_song_info_get_property (GObject *object,
 
 	switch (prop_id)
 	{
+	case PROP_SOURCE:
+		g_value_set_object (value, song_info->priv->source);
+		break;
 	case PROP_ENTRY_VIEW:
 		g_value_set_object (value, song_info->priv->entry_view);
 		break;
@@ -502,21 +499,25 @@ rb_song_info_get_property (GObject *object,
 }
 
 GtkWidget *
-rb_song_info_new (RBEntryView *entry_view)
+rb_song_info_new (RBSource *source, RBEntryView *entry_view)
 {
 	RBSongInfo *song_info;
 
-        g_return_val_if_fail (RB_IS_ENTRY_VIEW (entry_view), NULL);
+        g_return_val_if_fail (RB_IS_SOURCE (source), NULL);
+	if (!entry_view)
+		entry_view = rb_source_get_entry_view (source);
 
 	if (rb_entry_view_have_selection (entry_view) == FALSE) 
 		return NULL;
 
 	/* create the dialog */
-	song_info = g_object_new (RB_TYPE_SONG_INFO, "entry_view", entry_view, NULL);
+	song_info = g_object_new (RB_TYPE_SONG_INFO, 
+				  "source", source, 
+				  "entry-view", entry_view, 
+				  NULL);
 
 	g_return_val_if_fail (song_info->priv != NULL, NULL);
 
-	rb_song_info_populate_dialog (song_info);
 	return GTK_WIDGET (song_info);
 }
 
@@ -610,18 +611,91 @@ rb_song_info_populate_num_field (GtkEntry *field, gulong num)
 	g_free (tmp);
 }
 
+static void 
+rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
+{
+	gboolean mixed_artists = FALSE;
+	gboolean mixed_albums = FALSE;
+	gboolean mixed_genres = FALSE;
+	gboolean mixed_years = FALSE;
+	gboolean mixed_disc_numbers = FALSE;
+	const char *artist = NULL;
+	const char *album = NULL;
+	const char *genre = NULL;
+	int year = 0;
+	int disc_number = 0;
+	GList *l;
+
+	g_assert (song_info->priv->selected_entries);
+
+	for (l = song_info->priv->selected_entries; l != NULL; l = g_list_next (l)) {
+		RhythmDBEntry *entry;
+		const char *entry_artist;
+		const char *entry_album;
+		const char *entry_genre;
+		int entry_year;
+		int entry_disc_number;
+
+		entry = (RhythmDBEntry*)l->data;
+		entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+		entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
+		entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
+		entry_year = (entry->date) ? g_date_get_year (entry->date) : 0;
+		entry_disc_number = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
+
+		/* grab first valid values */
+		if (artist == NULL)
+			artist = entry_artist;
+		if (album == NULL)
+			album = entry_album;
+		if (genre == NULL)
+			genre = entry_genre;
+		if (year == 0)
+			year = entry_year;
+		if (disc_number == 0)
+			disc_number = entry_disc_number;
+
+		/* locate mixed values */
+		if (artist != entry_artist)
+			mixed_artists = TRUE;
+		if (album != entry_album)
+			mixed_albums = TRUE;
+		if (genre != entry_genre)
+			mixed_genres = TRUE;
+		if (year != entry_year)
+			mixed_years = TRUE;
+		if (disc_number != entry_disc_number)
+			mixed_disc_numbers = TRUE;
+
+		/* don't continue search if everything is mixed */
+		if (mixed_artists && mixed_albums && mixed_genres &&
+		    mixed_years && mixed_disc_numbers)
+			break;
+	}
+
+	if (!mixed_artists && artist != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), artist);
+	if (!mixed_albums && album != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), album);
+	if (!mixed_genres && genre != NULL)
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), genre);
+	if (!mixed_years && year != 0)
+		rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->year), year);
+	if (!mixed_disc_numbers && disc_number != 0)
+		rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur), disc_number);
+}
+
 static void 
 rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
 	const char *text = NULL;
 	char *tmp;
+	
+	g_assert (song_info->priv->current_entry);
+
 	/* update the buttons sensitivity */
 	rb_song_info_update_buttons (song_info);
 
-	if (!song_info->priv->current_entry)
-		return;
-	
-
 	text = rb_refstring_get (song_info->priv->current_entry->title);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
 	
@@ -766,9 +840,11 @@ static void
 rb_song_info_backward_clicked_cb (GtkWidget *button,
 				  RBSongInfo *song_info)
 {
-	song_info->priv->current_entry
-		= rb_entry_view_get_previous_from_entry (song_info->priv->entry_view,
-						     song_info->priv->current_entry);
+	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
+	song_info->priv->current_entry =
+		rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
+							      song_info->priv->current_entry);
+
 	rb_entry_view_select_entry (song_info->priv->entry_view,
 				    song_info->priv->current_entry);
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
@@ -781,9 +857,11 @@ static void
 rb_song_info_forward_clicked_cb (GtkWidget *button,
 				 RBSongInfo *song_info)
 {
-	song_info->priv->current_entry
-		= rb_entry_view_get_next_from_entry (song_info->priv->entry_view,
-						     song_info->priv->current_entry);
+	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
+	song_info->priv->current_entry =
+		rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
+							  song_info->priv->current_entry);
+
 	rb_entry_view_select_entry (song_info->priv->entry_view,
 				    song_info->priv->current_entry);
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
@@ -801,27 +879,32 @@ rb_song_info_update_buttons (RBSongInfo *song_info)
 	RhythmDBEntry *entry = NULL;
 
 	g_return_if_fail (song_info != NULL);
-	g_return_if_fail (song_info->priv->entry_view != NULL);
+	g_return_if_fail (song_info->priv->query_model != NULL);
 
 	if (!song_info->priv->current_entry)
 		return;
 	
 	/* backward */
-	entry = rb_entry_view_get_previous_from_entry (song_info->priv->entry_view,
-						       song_info->priv->current_entry);
+	entry = rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
+							      song_info->priv->current_entry);
 	
 	gtk_widget_set_sensitive (song_info->priv->backward, entry != NULL);
 	/* forward */
-	entry = rb_entry_view_get_next_from_entry (song_info->priv->entry_view,
-						   song_info->priv->current_entry);
+	entry = rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
+							  song_info->priv->current_entry);
 	
 	gtk_widget_set_sensitive (song_info->priv->forward, entry != NULL);
 }
 
 static void
-rb_song_info_view_changed_cb (RBEntryView *entry_view,
-			      RBSongInfo *song_info)
+rb_song_info_query_model_changed_cb (GObject *source,
+				     GParamSpec *whatever,
+				     RBSongInfo *song_info)
 {
+	if (song_info->priv->query_model)
+		g_object_unref (G_OBJECT (song_info->priv->query_model));
+
+	g_object_get (source, "query-model", &song_info->priv->query_model, NULL);
 	/* update next button sensitivity */
 	rb_song_info_update_buttons (song_info);
 }
@@ -877,8 +960,14 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
 	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
 	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	const char *year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
+	const char *discn_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
+
+	char *endptr;
 	GValue val = {0,};
 	GList *tem;
+	gint year;
+	gint discn;
 	gboolean changed = FALSE;
 	RhythmDBEntry *entry;
 
@@ -924,6 +1013,53 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_unset (&val);
 	}
 
+	if (strlen (year_str) > 0) {
+		GDate *date = NULL;
+		GType type;
+
+		/* note: this will reset the day-of-year to Jan 1 for all entries */
+		year = g_ascii_strtoull (year_str, &endptr, 10);
+		if (year > 0)
+			date = g_date_new_dmy (1, G_DATE_JANUARY, year);
+
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_DATE);
+
+		g_value_init (&val, type);
+		g_value_set_ulong (&val, (date ? g_date_get_julian (date) : 0));
+
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			entry = (RhythmDBEntry *)tem->data;
+			rhythmdb_entry_set (dialog->priv->db, entry,
+					    RHYTHMDB_PROP_DATE, &val);
+			changed = TRUE;
+		}
+		g_value_unset (&val);
+		if (date)
+			g_date_free (date);
+
+	}
+	
+	discn = g_ascii_strtoull (discn_str, &endptr, 10);
+	if (endptr != discn_str) {
+		GType type;
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_DISC_NUMBER);
+		g_value_init (&val, type);
+		g_value_set_ulong (&val, discn);
+
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			entry = (RhythmDBEntry *)tem->data;
+			if (discn != entry->discnum) {
+				rhythmdb_entry_set (dialog->priv->db, entry,
+						    RHYTHMDB_PROP_DISC_NUMBER, &val);
+				changed = TRUE;
+			}
+		}
+		g_value_unset (&val);
+	}
+
+
 	if (changed)
 		rhythmdb_commit (dialog->priv->db);
 }
@@ -1034,7 +1170,7 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 
 
-	if (strcmp (artist, rb_refstring_get (entry->artist))) {
+	if (strcmp (genre, rb_refstring_get (entry->genre))) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_GENRE);
 		g_value_init (&val, type);
